# frozen_string_literal: true

module SideBoom
  class Pipeline
    include SideBoom::Attributes

    class << self
      def define(sink = $stdout, &block)
        sink = File.open(sink, 'w+') if sink.is_a?(String)

        SideBoom::Context.in(self) do
          pipeline = new(sink)
          pipeline.instance_eval(&block)
          pipeline.write!
        end
      end
    end

    attr_reader :stages, :sink

    def initialize(sink)
      @sink = sink
      @stages = []
    end

    def stage(name, &block)
      SideBoom::Context.in(self) do
        new_stage = SideBoom::Stage.new(name)
        new_stage.instance_eval(&block) if block_given?
        @stages << new_stage
      end
    end

    def write!
      content = "# Generated by SideBoom\n" + to_yaml
      @sink.write(content)
      @sink.close if @sink.is_a?(File)
    end

    private

    def to_hash
      hsh = {}
      hsh['stages'] = @stages.map(&:name)

      @stages.each { |stage| hsh.merge!(stage.to_hash) }
      hsh
    end

    def to_yaml
      to_hash.to_yaml
    end
  end
end
